//  Copyright 2009 Stanislav Zidek
/*
 *  This file is part of CE-solver.
 *
 *  CE-solver is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CE-solver is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CE-solver.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef _CE_SOLVER_HH_
#define _CE_SOLVER_HH_
#include "ce-solver.hh"

#include "debug.hh"

#include "game-adt.hh"
#include "g-matrix.hh"
#include "ucache.hh"
#include "ce-misc.hh"
#include "ce-lp.hh"

//LP solvers interfaces
#include "glpk-solver.hh"
//#include "rs-solver.hh" //might be used instead of GLPK (strongly discouraged)

#include <map>

template <bool B, class T=float, class S=GLPKSolver<T> >
class CESolver {
private:
	const Game<T> &game;
	CorrelatedEquilibrium solution;
	UProfile<T> eq_u;
	bool solved;

public:
	CESolver(const Game<T>& g):
		game(g),
		solved(false)
	{}

	void solve(bool do_export=true, bool solve_lp=true)
	{
		GMatrix<B, T> g_matrix(game);
		g_matrix.minimize();


		if (do_export) {
			g_matrix.export_reduced();
			PRINTLN("Exported reduced game.");
		}

		if (solve_lp) {
			CorrelatedLP<T> clp = g_matrix.get_lp_problem();
			S solver(clp);

			solution = solver.solve();
			eq_u = g_matrix.get_payoffs(solution);

			solved = true;
		}

		return;
	}

	CorrelatedEquilibrium get_solution()
	{
		if (!solved) throw std::logic_error("Game is not solved.");
		DPRINT("Solution: ");
		typename CorrelatedEquilibrium::iterator it;
		for (it=solution.begin(); it!=solution.end(); it++) {
			DPRINT(it->first << ": " << it->second);
		}

		return solution;
	}

	UProfile<T> get_payoffs()
	{
		assert(solved);
		return eq_u;
	}
};

#endif //_CE_SOLVER_HH_
