//  Copyright 2009 Stanislav Zidek
/*
 *  This file is part of CE-solver.
 *
 *  CE-solver is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CE-solver is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CE-solver.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


#include "file-iface.hh"

#include <iostream>

void print_help()
{
	std::cout << "CE-solver (Correllated Equilibrium Solver) file interface, v 1.0" << std::endl;
	std::cout << "May 2009, Stanislav Zidek, FIT, BUT " << std::endl;
	std::cout << "USAGE:\ncefs [PARAMS] [FILE]" << std::endl;
	std::cout << "PARAMS:\t-h\tprint this help" << std::endl;
	std::cout << "\t-s\tsolve only, do not export reduced game" << std::endl;
	std::cout << "\t-e\tminimize only, do not solve game (only export reduced game)" << std::endl;
	std::cout << "\t-r name\tfilename of reduced game (exported to name.game and name.map)" << std::endl;
	std::cout << "\t\tdefault: \"reduced\"" << std::endl;
	std::cout << "FILE\tinput file" << std::endl;
	std::cout << "\tdefault: standard input" << std::endl;
}

int main(int argc, char *argv[])
{
	bool ok = true;
	char action = 'X';
	std::string file = "", reduced = "";
	for (int i=1; i<argc; i++) {
		std::string s = argv[i];
		if (s[0] == '-') {
			if (s.size() != 2) {
				ok = false;
				break;
			}
			char c = s[1];
			if (c == 'r') {
				if (i+1<argc && reduced == "") {
					i++;
					reduced = argv[i];
				} else {
					//missing reduced output name
					//or reduced again
					ok = false;
					break;
				}
			} else if (action != 'X') {
				//two actions
				ok = false;
				break;
			} else if (c=='h'||c=='s'||c=='e') {
				action = c;
			} else {
				//unknown action
				ok = false;
				break;
			}
		} else {
			if (file == "") {
				file = s;
			} else {
				//file again
				ok = false;
				break;
			}
		}
	}
	if (action == 'h' || !ok) {
		print_help();
		return !ok ? -1 : 0;
	}

	bool do_export = true, do_solve = true;
	if (action == 's') do_export = false;
	else if (action == 'e') do_solve = false;
	FileSolver<float> fs(file, reduced, do_export, do_solve);
	fs.solve();

	return 0;
}
