//  Copyright 2009 Stanislav Zidek
/*
 *  This file is part of CE-solver.
 *
 *  CE-solver is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CE-solver is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CE-solver.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

//GDA - Game Dominance Analyzer

#include "analyzer.hh"
#include "file-iface.hh"

#include <iostream>
#include <iomanip>

void print_help()
{
	std::cout << "GDA (Game Dominance Analyzer), v0.1" << std::endl;
	std::cout << "April 2009, Stanislav Zidek, FIT, BUT " << std::endl;
	std::cout << std::endl;
	std::cout << "USAGE:\ngda -h\t\t\tprint this help" << std::endl;
	std::cout << "gda <QUERY> [FILE]\tbatch mode" << std::endl;
	std::cout << "gda -i FILE\t\tinteractive mode" << std::endl;
	std::cout << std::endl;
	std::cout << "FILE\tinput file" << std::endl;
	std::cout << "\tdefault: standard input" << std::endl;
	std::cout << std::endl;
	std::cout << "QUERY\t[ACTION] RANGE : RANGE : RANGE (without spaces)" << std::endl;
	std::cout << std::endl;
	std::cout << "Range is for players : dominant strategies : dominated strategies" << std::endl;
	std::cout << "\tACTION\ta\tAnalyze (default)" << std::endl;
	std::cout << "\t\ts\t(select) Strictly dominant strategies" << std::endl;
	std::cout << "\t\tw\t(select) Weakly dominant strategies" << std::endl;
	std::cout << std::endl;
	std::cout << "\tRANGE\t<number>[,<number>]*\tenumeration" << std::endl;
	std::cout << "\t\t<number>-<number>\tinterval" << std::endl;
	std::cout << "\t\tx\t\t\tall possible values" << std::endl;
	std::cout << std::endl;
	std::cout << "\tEXAMPLES:" << std::endl;
	std::cout << std::endl;
	std::cout << "\ts0:3:x\t\tfor player 0, select all strategies, that are strictly dominated by strategy 3" << std::endl;
	std::cout << "\ts1,3:0-2:5,8\tfor players 1 and 3, select strategies from 0, 1 and 2 that weakly dominate strategy 5 or 8" << std::endl;
	std::cout << "\t2:x:x\t\tfor player 2, display statistics for all pairs of his strategies" << std::endl;
}

std::string read_line()
{
	std::string res = "";
	char c;
	while (std::cin >> c && std::cin.good()) {
		if (c != '\n') {
			res += c;
		} else {
			break;
		}
	}

	return res;
}

template<bool B>
void shell(Analyzer<B>& analyzer)
{
	std::cin >> std::noskipws;
	std::cout << "Welcome to query shell." << std::endl << std::flush;
	while (true) {
		std::cout << "$ " << std::flush;
		std::string line = read_line();
		//std::cout << "Line: " << line << std::endl;
		if (line == "quit") {
			break;
		}
		try {
			std::cout << analyzer.query(line);
		} catch (std::runtime_error e) {
			std::cout << "Error: " << e.what() << std::endl;
			std::cout << "Type \'quit\' to exit." << std::endl << std::flush;
		}
	}
	std::cin >> std::skipws;

	return;
}

int main(int argc, char *argv[])
{
	bool ok = true;
	bool interactive = false;
	bool help = false;
	std::string file = "";
	std::string query = "";
	if (argc == 1) {
		//interactive = true;
		ok = false;
	} else if (argc == 2) {
		std::string s = argv[1];
		if (s == "-h") help = true;
		else query = s;
	} else if (argc == 3) {
		std::string first = argv[1];
		std::string second = argv[2];
		if (first == "-i") {
			file = second;
			interactive = true;
		} else {
			query = first;
			file = second;
		}
	}

	if (help || !ok) {
		print_help();
		return 0;
	}

	FileGame<> fg;
	if (file == "") {
		fg.read(std::cin);
	} else {
		std::ifstream ifs(file.c_str());
		fg.read(ifs);
		ifs.close();
	}

	Analyzer<false> analyzer(fg);

	if (interactive) {
		shell(analyzer);
	} else {
		std::cout << analyzer.query(query);
	}

	return 0;
}
