//  Copyright 2009 Stanislav Zidek
/*
 *  This file is part of CE-solver.
 *
 *  CE-solver is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CE-solver is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CE-solver.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

//Example game - prisoner's dilemma

#include "ce-solver.hh"
#include "game-adt.hh"

template <class T>
class PrisonersDilemma: public Game<T> {
public:
	PrisonersDilemma() {}

	virtual unsigned players() const { return 2; }

	virtual unsigned strategies(unsigned player) const
	{
		return 2;
	}

	virtual StrategyProfile strategies() const
	{
		StrategyProfile sp(2);
		sp[0] = 2; sp[1] = 2;

		return sp;
	}

	virtual T get(const unsigned player, const StrategyProfile& sp) const
	{
		if (sp[0] == sp[1]) {
			if (sp[0] == 0) {
				return -1;
			} else {
				return -5;
			}
		} else {
			if (sp[player]) {
				return 0;
			} else {
				return -10;
			}
		}
	}

	virtual UProfile<T> get(const StrategyProfile& sp) const
	{
		UProfile<T> prof(sp.size());

		for (unsigned i=0; i<sp.size(); i++) {
			prof[i] = get(i, sp);
		}

		return prof;
	}
};

int main()
{
	PrisonersDilemma<float> game;
	CESolver<false> ces(game);
	ces.solve();

	return 0;
}
