//  Copyright 2009 Stanislav Zidek
/*
 *  This file is part of CE-solver.
 *
 *  CE-solver is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  CE-solver is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with CE-solver.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

//Implementation of LP solver by RS solver
//(we strongly suggest using GLPK instead)

#ifndef _RS_SOLVER_HH_
#define _RS_SOLVER_HH_
#include "rs-solver.hh"

#include "lp-solver.hh"
#include "game-adt.hh"
#include "ce-lp.hh"

#include <revised_two_phase.hh>

#include <vector>
#include <map>

#include <cstdio>

template <class T>
class RSSolver: public LPSolver<T> {
private:
	Problem<unsigned, T> problem;
	RevisedSimplexSolver<unsigned, T> rss;
public:
	RSSolver(const CorrelatedLP<T>& corr_lp):
		LPSolver<T>(corr_lp)
	{
		problem = Problem<unsigned, T>(corr_lp.get_rows()+1, corr_lp.get_cols());
		unsigned index = 0;
		for (unsigned i=0; i<corr_lp.get_rows(); i++) {
			std::vector<T> vec(corr_lp.get_cols(), 0.0);
			while (index < corr_lp.get_items()) {
				unsigned row, col;
				T val;
				corr_lp.get(index, row, col, val);
				if (row > i) break;
				assert(row == i);
				vec[col] = val;
				index++;
			}
			problem.add_constr(vec, 0.0, GE);
		}
		std::vector<T> vec(corr_lp.get_cols(), 1.0);
		problem.add_constr(vec, 1.0, EQ);

		problem.set_obj_func(corr_lp.get_obj_func(), MAX);

		rss = RevisedSimplexSolver<unsigned, T>(problem);
	}

	virtual std::map<StrategyProfile, T> solve()
	{
		rss.solve();
		std::map<unsigned, T> solution;
		bool res = rss.get_solution(solution);

		if (!res) {
			return std::map<StrategyProfile, T>();
		}

		return LPSolver<T>::clp.translate(solution);
	}
private:
};

#endif //_RS_SOLVER_HH_
